## ######### PROJECT NAME : ##########
## 
## raid_control.pm
## 
## ######### PROJECT DESCRIPTION : ###
## 
## Module de controle de la prsence et de l'tat des disques physiques
## 
## ###################################
# 3    UNKNOWN
# 2    CRITICAL
# 1    WARNING
# 0    OK

package raid_control;

use strict;
use Data::Dumper;
use raid;
use Objet_Alerte;   # Objet d envoie d email Intellique

use POSIX('strftime');

use Sys::Hostname;

# Chemin du fichier de configuration
my $file_conf   = "/etc/storiq/control.conf";

my %code_nagios  = (
    3   => "UNKNOWN",
    2   => "CRITICAL",
    1   => "WARNING",
    0   => "OK",
    -1  => "UNKNOW",
);

## ###################################
## Sortie nagios
sub formatage_sortie_nagios {
    my $list = shift;

    # Je boucle sur chaque etat nagios possible
    foreach my $i ( 2, 1, 3 ) {

        # Un ou plusieurs elements sont dans l'etat $i
        if ( defined $list->[ $i ] ) {
            my $msg;
            foreach my $element ( @{ $list->[ $i ] } ) {
	            $msg .= $code_nagios{ $i } .": ". $element->{"ctl"} ." : ";
	            $msg .= $element->{"port"} ." : " if ( $element->{"port"} );
	            $msg .= $element->{"message"} ."\n";
	        }
	
	        chomp $msg;
	        return $i, $msg;
	    }
    }

    # Tout va bien
    return 0, "OK";
}

## ###################################
## Envoie alerte

sub formatage_sortie_alerte {
    my ( $logger, $list ) = @_;

    my $max_level;
    my $max_error = 0;
    my $msg_mail;
    my %retour;
    
    my $hostname = hostname();
    my $serial = "";
    if ( open (FILES, "/usr/share/storiq/SerialNumber") ) {
        while (<FILES>) {
            $serial .= $_;
        }
        close FILES;
        chomp $serial;
    }

    my ($err, $alert ) = new Objet_Alerte("alerte_raid", $logger);
    return ($err, $alert ) if ( $err );

    # Je boucle sur chaque etat nagios possible
    foreach my $level ( 2, 1, 3 ) {

        # Un ou plusieurs elements sont dans l'etat $i
        if ( defined $list->[ $level ] ) {
            $max_level = $level unless ( defined $max_level );

            foreach my $element ( @{ $list->[ $level ] } ) {
                $msg_mail .= msg_mail( $element );
                my $msg = msg_snmp( $element );
                
                $logger->info( $msg );
                my ($err2, $send) = envoie_snmp( $alert, $level, $code_nagios{ $level }, $hostname, $serial, $msg );

                $max_error = $err2 if ( $max_error < $err2 );
                push @{ $retour{ $err2 } }, @{ $send };
	        }
	    }
    }

    if ( defined $max_level ) {
        chomp $msg_mail;
        
        my ($err2, $send) = envoie_mail( $alert, $code_nagios{ $max_level }, $hostname, $serial, $msg_mail);

        $max_error = $err2 if ( $max_error < $err2 );
        
        if ( ref $send eq "ARRAY" ) {
            push @{ $retour{ $err2 } }, @{ $send };
        }
        else {
            push @{ $retour{ $err2 } }, $send;
        }

        return $max_error, $retour{ $max_error };
    }

    $logger->info( $code_nagios{ "0" } );
    return $alert->send( [ [ "sujet", $code_nagios{ "0" }, $hostname, $serial], [ "message_ok", $hostname, $serial] ], [ "4.6.0", "OCTET_STRING", "0", "message_ok", $hostname, $serial]);
}

sub envoie_mail {
    my ( $alert, $code, $hostname, $serial, $msg_mail) = @_;
    
    return $alert->send( [ [ "sujet", $code, $hostname, $serial], [ "message", $hostname, $serial, $msg_mail ] ], undef);
}

sub envoie_snmp {
    my ( $alert, $level, $code, $hostname, $serial, $msg ) = @_;

    return $alert->send( undef, [ "4.6.3", "OCTET_STRING", $level, "message", $code, $hostname, $serial, $msg ]);
}


## ###################################
## Controle les elements

sub control_smart {
    my $logger = shift;
    my %control;

    if ( $_[0] ) {
        $control{ $_[0] } =  "";
        $control{ $_[0] } =  $_[1] if ( $_[1] );
    }
    else {
        #Objet Conf
        my ($err, $conf) = new Objet_Conf( $file_conf, $logger );
        return ( 3, "UNKNOWN: Pb creation Objet_Conf: $conf" ) if $err ;

        my ( $err2, $keys) = $conf->get_key();
        return ( 3, "UNKNOWN: Pb recuperation des cles: ". $keys ) if $err2 ;
        
        foreach my $ctl ( @{$keys} ) {
            my ($err3, $ref) = $conf->get_value( $ctl );
            return ( 3, "UNKNOWN: Pb recuperation des valeurs: ". $ref ) if $err3 ;
            
            $control{ $ctl } = $ref;
        }
    }

    return 3, "UNKNOWN: Il n'y a rien a controler\n" unless ( %control );

    my @list_retour;
    foreach my $ctl ( keys %control ) {
        # Teste pour controleur seul
        if ( ! $control{ $ctl } ) {
            my ( $etat_smart, $retour_smart )   = raid::smart( $ctl, "");
            return ($etat_smart, $retour_smart) if ( $etat_smart );

            my $level = raid_smart::translate_code_to_level( $retour_smart );
            push @{ $list_retour[ $level ] }, {
		                "port"      => "",
		                "ctl"       => $ctl,
		                "message"   => raid_smart::translate_code_to_text( $retour_smart ),
		            };
        }
        # Teste pour element sur controleur
        foreach my $port ( split( " ", $control{ $ctl } ) ) {
            my ( $etat_smart, $retour_smart )   = raid::smart( $ctl, $port);
            return ($etat_smart, $retour_smart) if ( $etat_smart );

            my $level = raid_smart::translate_code_to_level( $retour_smart );
            push @{ $list_retour[ $level ] }, {
		                "port"      => $port,
		                "ctl"       => $ctl,
		                "message"   => raid_smart::translate_code_to_text( $retour_smart ),
		            };
        }
    }

    return 0, @list_retour;
}

sub control {
    my $logger = shift;
    my $format = shift;

    # Si l'objet logger est absent, je le cre.
    if (!$logger) {
        undef, $logger = new Objet_Logger();
        $logger->debug( "control : $logger parametre is missing");
    }
    # Verification de l'integrite de l'objet Logger recu
    unless (ref($logger) eq "Objet_Logger") {
        undef, $logger = new Objet_Logger();
        $logger->debug( "control : $logger isn't an Objet_Logger");
    }


    my ($etat, @list_retour) = control_smart( $logger, @_ );
    return ( $etat, @list_retour) if ( $etat );

    # Envoie tout en alerte
    if ( $format eq "info" ) {
        return formatage_sortie_alerte( $logger, \@list_retour );
    }
    # Envoie tout en alerte
    if ( $format eq "ERROR" ) {
        return formatage_sortie_alerte( $logger, \@list_retour );
    }
    # Envoie tout en alerte
    if ( $format eq "alert" ) {
        return formatage_sortie_alerte( $logger, \@list_retour );
    }
    # Affichage Nagios et console
    return formatage_sortie_nagios( \@list_retour );
}

# Test l'envoie d'email et de trap SNMP et sort
sub test_alert {
    my $logger = shift;

    # Si l'objet logger est absent, je le cre.
    if (!$logger) {
        undef, $logger = new Objet_Logger();
        $logger->debug( "test_alert : $logger parametre is missing.");
    }
    # Verification de l'integrite de l'objet Logger recu
    unless (ref($logger) eq "Objet_Logger") {
        undef, $logger = new Objet_Logger();
        $logger->debug( "test_alert : $logger isn't an Objet_Logger");
    }

    my ($err, $alert ) = new Objet_Alerte("alerte_raid", $logger);
    return ($err, $alert ) if ( $err );

    my $hostname = hostname();
    my $serial = "";
    if ( open (FILES, "/usr/share/storiq/SerialNumber") ) {
        while (<FILES>) {
            $serial .= $_;
        }
        close FILES;
        chomp $serial;
    }

    $logger->info( "TEST" );
    return $alert->send( [ [ "sujet", "TEST", $hostname, $serial], [ "message_test", $hostname, $serial] ], [ "4.6.222", "OCTET_STRING", "0", "message_test", $hostname, $serial]);
}

# Fichier de conf comprenant tout ce qui est controlable
sub create_conf_file {
    my $logger = shift;

    # Si l'objet logger est absent, je le cre.
    if (!$logger) {
        undef, $logger = new Objet_Logger();
        $logger->debug( "create_conf_file : $logger parametre is missing.");
    }
    # Verification de l'integrite de l'objet Logger recu
    unless (ref($logger) eq "Objet_Logger") {
        undef, $logger = new Objet_Logger();
        $logger->debug( "create_conf_file : $logger isn't an Objet_Logger");
    }

    my %configuration;
    
    my ( $etat_ctl, $list_ctl ) = raid::list_ctl();
    
    foreach my $ctl ( @{$list_ctl} ) {
        my ( $etat_unit, $list_unit ) = raid::list_unit( $ctl );
        if ( ! $etat_unit and $list_unit ) {
            $configuration{ $ctl } = join ( " ", @{ $list_unit } );
        }
    
        next if ( $ctl eq "clo" );
        next if ( $ctl =~ m/^lvm/ );
        
        my ( $etat_port, $list_port ) = raid::list_port( $ctl );
        if ( ! $etat_port and $list_port ) {
            $configuration{ $ctl } .= " ". join ( " ", @{ $list_port } );
        }
        
        my ( $etat_bbu, $code_bbu ) = raid::smart( $ctl, "bbu" );
        if ( $code_bbu ne "11" ) {
            $configuration{ $ctl } .= " bbu";
        }
    }

    foreach ( sort keys %configuration ) {
        print "$_ = ". $configuration{ $_ } ."\n";
    }
    
    return 0, "";
}

sub msg_mail {
    my $element = shift;
    my $msg  = "Details :\n";
    $msg    .= "    Date : ". strftime("%Y/%m/%d %X",localtime) ."\n";
    $msg    .= "    Controller : ". $element->{"ctl"} ."\n";
    $msg    .= "    Element : ". $element->{"port"} ."\n" if ( $element->{"port"} );
    $msg    .= "    Information : ". $element->{"message"} ."\n\n";
    
    return $msg;
}
sub msg_snmp {
    my $element = shift;
    my $msg .= $element->{"ctl"} ." : ";
    $msg    .= $element->{"port"} ." : " if ( $element->{"port"} );
    $msg    .= $element->{"message"};
    return $msg;
}

1;
